"use strict";
exports.__esModule = true;
var util_1 = require("./jira/util");
var Parser = /** @class */ (function () {
    function Parser() {
    }
    Parser.parse = function (text) {
        var _this = this;
        var days = text
            .split("#")
            .map(function (d) { return d.trim(); })
            .filter(function (v) { return !!v; });
        return days.map(function (day) {
            var first = day.split("\n")[0].trim();
            var date = util_1.parseDate(first, ["d.M.", "d.M.yy"]);
            if (!date) {
                console.error("Could not parse date " + first);
                process.exit(1);
            }
            return {
                date: date,
                workEntries: _this.parseDay(day)
            };
        });
    };
    Parser.parseDay = function (text) {
        var entries = [];
        var lines = text
            .split("\n")
            .slice(1)
            .map(function (t) { return t.trim(); });
        var startTime = null;
        var endTime = null;
        var ticketId = null;
        var description = null;
        for (var i = 0; i < lines.length; i += 1) {
            if (!startTime) {
                startTime = this.parseTime(lines[i]);
                continue;
            }
            if (!description || !ticketId) {
                var parts = lines[i].split(":");
                // Add existing description if just adding something to a worklog
                if (parts.length === 1) {
                    ticketId = parts[0].replace("+", "").trim();
                    var existingEntry = entries.find(function (e) { return e.ticketId === ticketId; });
                    if (!existingEntry) {
                        console.error("Missing Description for " + ticketId + " on day " + startTime);
                        process.exit(1);
                    }
                    description = existingEntry.description;
                }
                else {
                    ticketId = parts[0].replace("+", "").trim();
                    description = parts
                        .slice(1)
                        .join(":")
                        .trim();
                }
                continue;
            }
            endTime = this.parseTime(lines[i]);
            entries.push({
                startTime: startTime,
                endTime: endTime,
                ticketId: ticketId,
                description: description
            });
            // end time is start time of next
            startTime = endTime;
            // Take breaks into account
            if (i < lines.length - 1 &&
                lines[i + 1].replace("+", "").trim().length < 6) {
                startTime = null;
            }
            endTime = ticketId = description = null;
        }
        return entries;
    };
    Parser.parseTime = function (text) {
        var trimmed = text.replace("+", "").trim();
        var time = util_1.parseDate(trimmed, ["H", "H:m"]);
        if (!time) {
            console.error("Time " + trimmed + " not in right format");
            process.exit(1);
        }
        return time;
    };
    return Parser;
}());
exports.Parser = Parser;
